@@ -145,7 +145,7 @@ void add_query_param( struct qserver *server, char *arg);
  * @param default_value value to return if key was not found
  * @return value for key or default_value
  */
-char * get_param_value( struct qserver *server, char *key, char *default_value);
+char * get_param_value( struct qserver *server, const char *key, char *default_value);
 
 /** @see get_param_value */
 int get_param_i_value( struct qserver *server, char *key, int default_value);
@@ -176,4 +176,6 @@ int qserver_send_initial(struct qserver* server, const char* data, size_t len);
  */
 int qserver_send(struct qserver* server, const char* data, size_t len);
 
+int send_broadcast( struct qserver *server, const char *pkt, size_t pktlen);
+
 #endif
