@@ -14,11 +14,6 @@
 #include "gnuconfig.h"
 #endif
 
-#ifdef _WIN32
-#else
-#define _ISUNIX
-#endif
-
 #ifdef __EMX__
 #include <sys/select.h>
 #include <sys/param.h>
@@ -26,12 +21,6 @@
 #define strncasecmp strnicmp
 #endif
 
-
-#ifdef _ISUNIX
-#include <sys/time.h>
-#define SOCKET_ERROR -1
-#endif
-
 #ifdef _WIN32
 # include <sys/timeb.h>
 # include <winsock.h>
@@ -58,6 +47,9 @@ static int gettimeofday(struct timeval *now, void *blah)
 #  define EADDRINUSE	WSAEADDRINUSE
 # endif
 # define snprintf _snprintf
+#else
+# include <sys/time.h>
+# define SOCKET_ERROR -1
 #endif /* _WIN32 */
 
 #include <string.h>
@@ -65,10 +57,18 @@ static int gettimeofday(struct timeval *now, void *blah)
 typedef struct _server_type server_type;
 
 #include "qserver.h"
-#include "ut2004.h"
-#include "a2s.h"
 
-/* Various magic numbers.
+// Packet modules
+#include "ut2004.h"
+#include "doom3.h"
+#include "a2s.h"
+#include "gps.h"
+#include "gs2.h"
+#include "gs3.h"
+#include "ts2.h"
+
+/*
+ * Various magic numbers.
  */
 
 #define Q_DEFAULT_PORT			26000
@@ -109,10 +109,9 @@ typedef struct _server_type server_type;
 #define SAVAGE_DEFAULT_PORT	11235
 #define FARCRY_DEFAULT_PORT	49001
 #define STEAM_MASTER_DEFAULT_PORT	27010
-#define DOOM3_DEFAULT_PORT	27666
-#define DOOM3_MASTER_DEFAULT_PORT	27650
 #define HL2_DEFAULT_PORT	27015
 #define HL2_MASTER_DEFAULT_PORT	27011
+#define TS2_DEFAULT_PORT 51234
 
 
 #define Q_UNKNOWN_TYPE 0
@@ -167,8 +166,13 @@ typedef struct _server_type server_type;
 #define HL2_MASTER (47|MASTER_SERVER)
 #define UT2004_MASTER (48|MASTER_SERVER)
 #define A2S_SERVER	49
+#define PARIAH_SERVER	50
+#define GAMESPY3_PROTOCOL_SERVER 51
+#define TS2_PROTOCOL_SERVER 52
+#define QUAKE4_SERVER 53
+#define QUAKE4_MASTER (53|MASTER_SERVER)
 
-#define LAST_BUILTIN_SERVER  49
+#define LAST_BUILTIN_SERVER  53
 
 #define TF_SINGLE_QUERY		(1<<1)
 #define TF_OUTFILE		(1<<2)
@@ -190,6 +194,12 @@ typedef struct _server_type server_type;
 
 #define TF_MASTER_STEAM		(1<<16)  /* supports steam server filter */
 
+// What response type are we expecting
+// XXX: this is not what server->flags is for
+#define TF_STATUS_QUERY		(1<<17)
+#define TF_PLAYER_QUERY		(1<<18)
+#define TF_RULES_QUERY		(1<<19)
+
 #define TRIBES_TEAM	-1
 
 struct q_packet;
@@ -198,7 +208,8 @@ typedef void (*DisplayFunc)( struct qserver *);
 typedef void (*QueryFunc)( struct qserver *);
 typedef void (*PacketFunc)( struct qserver *, char *rawpkt, int pktlen);
 
-/* Output and formatting functions
+/*
+ * Output and formatting functions
  */
 
 void display_server( struct qserver *server);
@@ -223,6 +234,7 @@ void display_eye_player_info( struct qserver *server);
 void display_gs2_player_info( struct qserver *server);
 void display_doom3_player_info( struct qserver *server);
 void display_hl2_player_info( struct qserver *server);
+void display_ts2_player_info( struct qserver *server);
 
 void raw_display_server( struct qserver *server);
 void raw_display_server_rules( struct qserver *server);
@@ -244,6 +256,7 @@ void raw_display_eye_player_info( struct qserver *server);
 void raw_display_gs2_player_info( struct qserver *server);
 void raw_display_doom3_player_info( struct qserver *server);
 void raw_display_hl2_player_info( struct qserver *server);
+void raw_display_ts2_player_info( struct qserver *server);
 
 void xml_display_server( struct qserver *server);
 void xml_header();
@@ -267,13 +280,13 @@ void xml_display_eye_player_info( struct qserver *server);
 void xml_display_gs2_player_info( struct qserver *server);
 void xml_display_doom3_player_info( struct qserver *server);
 void xml_display_hl2_player_info( struct qserver *server);
+void xml_display_ts2_player_info( struct qserver *server);
 char *xml_escape( char*);
 char *str_replace( char *, char *, char *);
 
 void send_server_request_packet( struct qserver *server);
 void send_qserver_request_packet( struct qserver *server);
 void send_qwserver_request_packet( struct qserver *server);
-void send_unreal_request_packet( struct qserver *server);
 void send_ut2003_request_packet( struct qserver *server);
 void send_tribes_request_packet( struct qserver *server);
 void send_qwmaster_request_packet( struct qserver *server);
@@ -291,6 +304,7 @@ void send_eye_request_packet( struct qserver *server);
 void send_gs2_request_packet( struct qserver *server);
 void send_doom3_request_packet( struct qserver *server);
 void send_hl2_request_packet( struct qserver *server);
+void send_ts2_request_packet( struct qserver *server);
 
 void deal_with_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_q_packet( struct qserver *server, char *pkt, int pktlen);
@@ -301,7 +315,6 @@ void deal_with_q2_packet( struct qserver *server, char *pkt, int pktlen,
 void deal_with_doom3master_packet( struct qserver *server, char *rawpkt, int pktlen);
 void deal_with_qwmaster_packet( struct qserver *server, char *pkt, int pktlen);
 int deal_with_halflife_packet( struct qserver *server, char *pkt, int pktlen);
-void deal_with_unreal_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_ut2003_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_tribes_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_tribesmaster_packet( struct qserver *server, char *pkt, int pktlen);
@@ -316,9 +329,9 @@ void deal_with_descent3_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_descent3master_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_ghostrecon_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_eye_packet( struct qserver *server, char *pkt, int pktlen);
-void deal_with_gs2_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_doom3_packet( struct qserver *server, char *pkt, int pktlen);
 void deal_with_hl2_packet( struct qserver *server, char *pkt, int pktlen);
+void deal_with_ts2_packet( struct qserver *server, char *pkt, int pktlen);
 
 struct _server_type  {
     int id;
@@ -459,6 +472,9 @@ char ut2003_basicstatus[] = { 0x78, 0,0,0, 0 };
 //char ut2003_playerinfo[] = { 0x78, 0,0,0, 2 };
 char ut2003_allinfo[] = { 0x78, 0,0,0, 3 };
 
+/* Pariah */
+char pariah_basicstatus[] = { 0x77, 0,0,0, 0x13 };
+
 /* HALF LIFE */
 char hl_ping[9] =
 { '\377', '\377', '\377', '\377', 'p', 'i', 'n', 'g', '\0' };
@@ -483,9 +499,6 @@ char q3_master_query_template[] = "\377\377\377\377getservers %s %s";
 char q3_master_default_protocol[] = "68";
 char q3_master_default_query[] = "empty full demo\n";
 
-char doom3_master_query[] = "\xFF\xFFgetServers\x00\x21\x00\x01\x00\x00";
-//                                             ^^^^^^^^^^^^^^^^ version
-
 /* RETURN TO CASTLE WOLFENSTEIN */
 char rtcw_master_default_protocol[] = "60";
 
@@ -602,14 +615,52 @@ unsigned char ghostrecon_playerquery[] = {
 char eye_status_query[1]= "s";
 char eye_ping_query[1]= "p";
 
-// Gamespy v2 last 3 bytes:
-// 1: server + rules info (00 to disable)
-// 2: Player information (00 to disable)
-// 3: Team information (00 to disable)
+// Gamespy v2
+// Format:
+// 1 - 3: query head
+// 4 - 7: queryid
+// 8: server + rules info (00 to disable)
+// 9: Player information (00 to disable)
+// 10: Team information (00 to disable)
 unsigned char gs2_status_query[] = {
 	0xfe,0xfd,0x00,0x10,0x20,0x30,0x40,0xff,0xff,0xff
 };
 
+// Gamespy v3
+// Format:
+// 1 - 3: query head
+// 4 - 7: queryid
+// 8: server + rules info (00 to disable)
+// 9: Player information (00 to disable)
+// 10: Team information (00 to disable)
+// 11: Request new format
+unsigned char gs3_player_query[] = {
+	0xfe,0xfd,0x00,0x10,0x20,0x30,0x40,0xff,0xff,0xff,0x01
+};
+
+// Format:
+// 1 - 3: query head
+// 4 - 7: queryid
+// 8: requested number of rules
+// 9 - 9 + no_rules: requested ruleid
+// last 2 : terminator?
+
+// Known ruleid's:
+// 0x01: hostname
+// 0x03: version
+// 0x04: hostport
+// 0x05: map
+// 0x06: gametype
+// 0x07: gamevarient
+// 0x08: num_players
+// 0x0a: max_players
+// 0x0b: gamemode
+unsigned char gs3_status_query[] = {
+	0xfe,0xfd,0x00,
+	0x10,0x20,0x30,0x40,
+	0x06,0x01,0x06,0x05,0x08,0x0a,0x04,0x00,0x00
+};
+
 // Steam
 // Format:
 // 1. Request type ( 1 byte )
@@ -665,6 +716,8 @@ unsigned char farcry_serverquery[] = {
 
 char ravenshield_serverquery[] = "REPORT";
 
+unsigned char ts2_status_query[] = "si";
+
 server_type builtin_types[] = {
 {
     /* QUAKE */
@@ -1103,10 +1156,10 @@ server_type builtin_types[] = {
     raw_display_server_rules,	/* display_raw_rule_func */
     xml_display_unreal_player_info,	/* display_xml_player_func */
     xml_display_server_rules,	/* display_xml_rule_func */
-    send_unreal_request_packet,	/* status_query_func */
+    send_gps_request_packet,	/* status_query_func */
     NULL,			/* rule_query_func */
     NULL,			/* player_query_func */
-    deal_with_unreal_packet,	/* packet_func */
+    deal_with_gps_packet,	/* packet_func */
 },
 {
     /* HALF LIFE */
@@ -1206,10 +1259,10 @@ server_type builtin_types[] = {
     raw_display_server_rules,	/* display_raw_rule_func */
     xml_display_q2_player_info,	/* display_xml_player_func */
     xml_display_server_rules,	/* display_xml_rule_func */
-    send_unreal_request_packet,	/* status_query_func */
+    send_gps_request_packet,	/* status_query_func */
     NULL,			/* rule_query_func */
     NULL,			/* player_query_func */
-    deal_with_unreal_packet,	/* packet_func */
+    deal_with_gps_packet,	/* packet_func */
 },
 {
     /* TRIBES */
@@ -1391,7 +1444,7 @@ server_type builtin_types[] = {
     0,				/* master */
     0,				/* default_port */
     0,				/* port_offset */
-    TF_SINGLE_QUERY,		/* flags */
+    TF_SINGLE_QUERY|TF_U2_NAMES,		/* flags */
     "gametype",			/* game_rule */
     "GAMESPYPROTOCOL",		/* template_var */
     (char*) &unreal_serverstatus,	/* status_packet */
@@ -1410,10 +1463,10 @@ server_type builtin_types[] = {
     raw_display_server_rules,	/* display_raw_rule_func */
     xml_display_unreal_player_info,	/* display_xml_player_func */
     xml_display_server_rules,	/* display_xml_rule_func */
-    send_unreal_request_packet,	/* status_query_func */
+    send_gps_request_packet,	/* status_query_func */
     NULL,			/* rule_query_func */
     NULL,			/* player_query_func */
-    deal_with_unreal_packet,	/* packet_func */
+    deal_with_gps_packet,	/* packet_func */
 },
 {
     /* TRIBES 2 */
@@ -1478,10 +1531,10 @@ server_type builtin_types[] = {
     raw_display_server_rules,	/* display_raw_rule_func */
     xml_display_descent3_player_info,	/* display_xml_player_func */
     xml_display_server_rules,	/* display_xml_rule_func */
-    send_unreal_request_packet,	/* status_query_func */
+    send_gps_request_packet,	/* status_query_func */
     NULL,			/* rule_query_func */
     NULL,			/* player_query_func */
-    deal_with_unreal_packet,	/* packet_func */
+    deal_with_gps_packet,	/* packet_func */
 },
 {
     /* DESCENT3 PROTOCOL */
@@ -1512,7 +1565,7 @@ server_type builtin_types[] = {
     raw_display_server_rules,	/* display_raw_rule_func */
     xml_display_descent3_player_info,	/* display_xml_player_func */
     xml_display_server_rules,	/* display_xml_rule_func */
-    send_unreal_request_packet,	/* status_query_func */
+    send_gps_request_packet,	/* status_query_func */
     NULL,			/* rule_query_func */
     NULL,			/* player_query_func */
     deal_with_descent3_packet,	/* packet_func */
@@ -1546,7 +1599,7 @@ server_type builtin_types[] = {
     raw_display_server_rules,	/* display_raw_rule_func */
     xml_display_descent3_player_info,	/* display_xml_player_func */
     xml_display_server_rules,	/* display_xml_rule_func */
-    send_unreal_request_packet,	/* status_query_func */
+    send_gps_request_packet,	/* status_query_func */
     NULL,			/* rule_query_func */
     NULL,			/* player_query_func */
     deal_with_descent3_packet,	/* packet_func */
@@ -1594,7 +1647,7 @@ server_type builtin_types[] = {
     "All Seeing Eye Protocol",	/* game_name */
     0,				/* master */
     0,				/* default_port */
-    0,			/* port_offset */
+    123,			/* port_offset */
     TF_SINGLE_QUERY,		/* flags */
     "gametype",				/* game_rule */
     "EYEPROTOCOL",		/* template_var */
@@ -1757,7 +1810,6 @@ server_type builtin_types[] = {
 },
 
 
-
 /* --- MASTER SERVERS --- */
 {
     /* QUAKE WORLD MASTER */
@@ -1880,8 +1932,8 @@ server_type builtin_types[] = {
     0,				/* player_len */
     NULL,			/* rule_packet */
     0,				/* rule_len */
-    doom3_master_query,	/* master_packet */
-    sizeof(doom3_master_query),/* master_len */
+    NULL,	/* master_packet */
+    0,          /* master_len */
     NULL,	/* master_protocol */
     NULL,	/* master_query */
     display_qwmaster,		/* display_player_func */
@@ -1890,7 +1942,7 @@ server_type builtin_types[] = {
     NULL,	/* display_raw_rule_func */
     NULL,	/* display_xml_player_func */
     NULL,	/* display_xml_rule_func */
-    send_qwmaster_request_packet,/* status_query_func */
+    send_doom3master_request_packet,/* status_query_func */
     NULL,			/* rule_query_func */
     NULL,			/* player_query_func */
     deal_with_doom3master_packet,	/* packet_func */
@@ -2270,6 +2322,176 @@ server_type builtin_types[] = {
     NULL,				/* player_query_func */
     deal_with_a2s_packet,		/* packet_func */
 },
+{
+    /* PARIAH */
+    PARIAH_SERVER,	/* id */
+    "PRS",			/* type_prefix */
+    "prs",			/* type_string */
+    "-prs",			/* type_option */
+    "Pariah",		/* game_name */
+    0,				/* master */
+    UNREAL_DEFAULT_PORT,	/* default_port */
+    1,				/* port_offset */
+    TF_U2_NAMES,				/* flags */
+    "gametype",			/* game_rule */
+    "UNREALTOURNAMENT2003",	/* template_var */
+    (char*) &pariah_basicstatus,/* status_packet */
+    sizeof( pariah_basicstatus),/* status_len */
+    NULL,			/* player_packet */
+    0,				/* player_len */
+    NULL,			/* rule_packet */
+    0,				/* rule_len */
+    NULL,			/* master_packet */
+    0,				/* master_len */
+    NULL,			/* master_protocol */
+    0,				/* master_query */
+    display_unreal_player_info,	/* display_player_func */
+    display_server_rules,	/* display_rule_func */
+    raw_display_unreal_player_info,	/* display_raw_player_func */
+    raw_display_server_rules,	/* display_raw_rule_func */
+    xml_display_unreal_player_info,	/* display_xml_player_func */
+    xml_display_server_rules,	/* display_xml_rule_func */
+    send_ut2003_request_packet,	/* status_query_func */
+    NULL,			/* rule_query_func */
+    NULL,			/* player_query_func */
+    deal_with_ut2003_packet,	/* packet_func */
+},
+{
+    /* GAMESPY V3 PROTOCOL */
+    GAMESPY3_PROTOCOL_SERVER,	/* id */
+    "GS3",			/* type_prefix */
+    "gs3",			/* type_string */
+    "-gs3",			/* type_option */
+    "Gamespy V3 Protocol",	/* game_name */
+    0,				/* master */
+    0,				/* default_port */
+    0,				/* port_offset */
+    TF_SINGLE_QUERY,		/* flags */
+    "gametype",			/* game_rule */
+    "GPS3PROTOCOL",		/* template_var */
+    (char*) &gs3_status_query,	/* status_packet */
+    sizeof( gs3_status_query),	/* status_len */
+    (char*) &gs3_player_query,	/* player_packet */
+    sizeof( gs3_player_query),	/* player_len */
+    NULL,			/* rule_packet */
+    0,				/* rule_len */
+    NULL,			/* master_packet */
+    0,				/* master_len */
+    NULL,			/* master_protocol */
+    NULL,			/* master_query */
+    display_gs2_player_info,	/* display_player_func */
+    display_server_rules,	/* display_rule_func */
+    raw_display_gs2_player_info,	/* display_raw_player_func */
+    raw_display_server_rules,	/* display_raw_rule_func */
+    xml_display_gs2_player_info,	/* display_xml_player_func */
+    xml_display_server_rules,	/* display_xml_rule_func */
+    send_gs3_request_packet,	/* status_query_func */
+    NULL,			/* rule_query_func */
+    NULL,			/* player_query_func */
+    deal_with_gs3_packet,	/* packet_func */
+},
+{
+    /* TEAMSPEAK 2 PROTOCOL */
+    TS2_PROTOCOL_SERVER,			/* id */
+    "TS2",							/* type_prefix */
+    "ts2",							/* type_string */
+    "-ts2",							/* type_option */
+    "Teamspeak 2",					/* game_name */
+    0,								/* master */
+    0,								/* default_port */
+    0,								/* port_offset */
+    TF_TCP_CONNECT|TF_QUERY_ARG_REQUIRED|TF_QUERY_ARG,	/* flags */
+    "N/A",							/* game_rule */
+    "TS2PROTOCOL",					/* template_var */
+    NULL,							/* status_packet */
+    0,								/* status_len */
+    NULL,							/* player_packet */
+    0,								/* player_len */
+    NULL,							/* rule_packet */
+    0,								/* rule_len */
+    NULL,							/* master_packet */
+    0,								/* master_len */
+    NULL,							/* master_protocol */
+    NULL,							/* master_query */
+    display_ts2_player_info,		/* display_player_func */
+    display_server_rules,			/* display_rule_func */
+    raw_display_ts2_player_info,	/* display_raw_player_func */
+    raw_display_server_rules,		/* display_raw_rule_func */
+    xml_display_ts2_player_info,	/* display_xml_player_func */
+    xml_display_server_rules,		/* display_xml_rule_func */
+    send_ts2_request_packet,		/* status_query_func */
+    NULL,							/* rule_query_func */
+    NULL,							/* player_query_func */
+    deal_with_ts2_packet,			/* packet_func */
+},
+{
+    /* QUAKE 4 */
+    QUAKE4_SERVER,					/* id */
+    "Q4S",							/* type_prefix */
+    "q4s",							/* type_string */
+    "-q4s",						/* type_option */
+    "Quake 4",						/* game_name */
+    0,								/* master */
+    QUAKE4_DEFAULT_PORT,				/* default_port */
+    0,								/* port_offset */
+    TF_QUAKE3_NAMES,				/* flags */
+    "fs_game",						/* game_rule */
+    "QUAKE4",						/* template_var */
+    (char*) &doom3_serverinfo,		/* status_packet */
+    sizeof( doom3_serverinfo),		/* status_len */
+    NULL,							/* player_packet */
+    0,								/* player_len */
+    NULL,							/* rule_packet */
+    0,								/* rule_len */
+    NULL,							/* master_packet */
+    0,								/* master_len */
+    NULL,							/* master_protocol */
+    NULL,							/* master_query */
+    display_doom3_player_info,		/* display_player_func */
+    display_server_rules,			/* display_rule_func */
+    raw_display_doom3_player_info,	/* display_raw_player_func */
+    raw_display_server_rules,		/* display_raw_rule_func */
+    xml_display_doom3_player_info,	/* display_xml_player_func */
+    xml_display_server_rules,		/* display_xml_rule_func */
+    send_qwserver_request_packet,	/* status_query_func */
+    NULL,							/* rule_query_func */
+    NULL,							/* player_query_func */
+    deal_with_quake4_packet,			/* packet_func */
+},
+{
+    /* QUAKE 4 MASTER */
+    QUAKE4_MASTER,		/* id */
+    "Q4M",			/* type_prefix */
+    "q4m",			/* type_string */
+    "-q4m",			/* type_option */
+    "Quake 4 Master",		/* game_name */
+    QUAKE4_SERVER,		/* master */
+    QUAKE4_MASTER_DEFAULT_PORT,	/* default_port */
+    0,				/* port_offset */
+    TF_OUTFILE|TF_QUERY_ARG,	/* flags */
+    "",				/* game_rule */
+    "QUAKE4MASTER",			/* template_var */
+    NULL,			/* status_packet */
+    0,				/* status_len */
+    NULL,			/* player_packet */
+    0,				/* player_len */
+    NULL,			/* rule_packet */
+    0,				/* rule_len */
+    NULL,	/* master_packet */
+    0,          /* master_len */
+    NULL,	/* master_protocol */
+    NULL,	/* master_query */
+    display_qwmaster,		/* display_player_func */
+    NULL,	/* display_rule_func */
+    NULL,	/* display_raw_player_func */
+    NULL,	/* display_raw_rule_func */
+    NULL,	/* display_xml_player_func */
+    NULL,	/* display_xml_rule_func */
+    send_quake4master_request_packet,/* status_query_func */
+    NULL,			/* rule_query_func */
+    NULL,			/* player_query_func */
+    deal_with_doom3master_packet,	/* packet_func */
+},
 {
     Q_UNKNOWN_TYPE,		/* id */
     "",				/* type_prefix */
@@ -2335,7 +2557,7 @@ struct player  {
     short flags;
     short type_flag;	/* Tribes 2 only */
     int packet_loss;	/* Tribes only */
-    char *tribe_tag;	/* Tribes 2 only */
+    char *tribe_tag;	/* Tribes 2 / Quake 4 clan name */
     char *skin;
     char *mesh;		/* Unreal only */
     char *face;		/* Unreal only */
@@ -2398,6 +2620,7 @@ extern int show_errors;
 
 extern int get_player_info;
 extern int get_server_rules;
+extern int no_port_offset;
 
 /* Definitions for the original Quake network protocol.
  */
@@ -2470,17 +2693,16 @@ char *ping_time( int ms);
 char *get_qw_game( struct qserver *server);
 
 
-/* Query status and packet handling functions
+/*
+ * Query status and packet handling functions
  */
 
 int cleanup_qserver( struct qserver *server, int force);
+void change_server_port( struct qserver *server, unsigned short port);
 
-int server_info_packet( struct qserver *server, struct q_packet *pkt,
-        int datalen);
-int player_info_packet( struct qserver *server, struct q_packet *pkt,
-        int datalen);
-int rule_info_packet( struct qserver *server, struct q_packet *pkt,
-	int datalen);
+int server_info_packet( struct qserver *server, struct q_packet *pkt, int datalen );
+int player_info_packet( struct qserver *server, struct q_packet *pkt, int datalen );
+int rule_info_packet( struct qserver *server, struct q_packet *pkt, int datalen );
 
 int time_delta( struct timeval *later, struct timeval *past);
 char * strherror( int h_err);
@@ -2488,8 +2710,7 @@ int connection_refused();
 
 void add_file( char *filename);
 int add_qserver( char *arg, server_type* type, char *outfilename, char *query_arg);
-struct qserver* add_qserver_byaddr( unsigned int ipaddr, unsigned short port,
-	server_type* type, int *new_server);
+struct qserver* add_qserver_byaddr( unsigned int ipaddr, unsigned short port, server_type* type, int *new_server);
 void init_qserver( struct qserver *server, server_type* type);
 int bind_qserver( struct qserver *server);
 int bind_sockets();
@@ -2497,8 +2718,6 @@ void send_packets();
 struct qserver * find_server_by_address( unsigned int ipaddr, unsigned short port);
 void add_server_to_hash( struct qserver *server);
 
-
-
 void print_packet( struct qserver *server, char *buf, int buflen);
 
 #define NO_FLAGS 0
@@ -2507,8 +2726,11 @@ void print_packet( struct qserver *server, char *buf, int buflen);
 #define NO_KEY_COPY 4
 #define COMBINE_VALUES 8
 
-struct rule* add_rule( struct qserver *server, char *key, char *value,	int flags);
+struct player* get_player_by_number( struct qserver *server, int player_number );
+struct rule* add_rule( struct qserver *server, char *key, char *value,	int flags) ;
 struct player* add_player( struct qserver *server, int player_number );
+struct info* player_add_info( struct player *player, char *key, char *value, int flags );
+void players_set_teamname( struct qserver *server, int teamid, char *teamname );
 
 
 /*
@@ -2554,4 +2776,13 @@ float swap_float_from_little( void *f);
 /** \brief write four bytes in little endian order */
 void put_long_little(unsigned val, char* buf);
 
+/*
+ * Exported Globals
+ */
+extern int show_game_port;
+
+#define NA_INT -32767
+#define NO_PLAYER_INFO 0xffff
+#define NO_SERVER_RULES NULL
+
 #endif
